@@ -19,6 +19,7 @@
 #pragma once
 
 #include <qlat-utils/show.h>
+#include <qlat-utils/assert.h>
 
 #include <sys/time.h>
 #include <algorithm>
@@ -26,11 +27,13 @@
 #include <cstdio>
 #include <cstdlib>
 #include <cstring>
+#include <cmath>
 #include <string>
 #include <vector>
+#include <map>
+#include <set>
 
 #ifdef USE_PAPI
-#include <omp.h>
 #include <papi.h>
 #endif
 
@@ -282,6 +285,20 @@ inline long long get_total_flops()
   return flops;
 }
 
+template <class K>
+bool has(const std::set<K>& m, const K& key)
+{
+  typename std::set<K>::const_iterator it = m.find(key);
+  return it != m.end();
+}
+
+template <class K, class M>
+bool has(const std::map<K, M>& m, const K& key)
+{
+  typename std::map<K, M>::const_iterator it = m.find(key);
+  return it != m.end();
+}
+
 API inline void initialize_papi()
 {
 #ifdef USE_PAPI
@@ -305,6 +322,8 @@ struct API TimerInfo {
   long long accumulated_flops;
   int call_times;
   //
+  TimerInfo() { init(); }
+  //
   void init()
   {
     fname = "Unknown";
@@ -394,13 +413,19 @@ struct API Timer {
   const char* cname;
   long info_index;
   bool is_using_total_flops;
-  long isRunning;
+  long is_running;
   double start_time;
   double stop_time;
   long long start_flops;
   long long stop_flops;
   long long flops;
   //
+  API static std::map<std::string, long>& get_timer_info_index_map()
+  {
+    static std::map<std::string, long> timer_info_index_map;
+    return timer_info_index_map;
+  }
+  //
   API static std::vector<TimerInfo>& get_timer_database()
   {
     static std::vector<TimerInfo> timer_database;
@@ -412,11 +437,13 @@ struct API Timer {
     static std::vector<std::vector<TimerInfo> > timer_database_history;
     return timer_database_history;
   }
+  //
   API static std::vector<double>& get_start_time_history()
   {
     static std::vector<double> history;
     return history;
   }
+  //
   API static std::vector<long>&
   get_max_call_times_for_always_show_info_history()
   {
@@ -481,18 +508,18 @@ struct API Timer {
   API static void merge()
   // call merge only after fork
   {
-    pqassert(get_start_time_history().size() >= 1);
+    qassert(get_start_time_history().size() >= 1);
     get_start_time() = get_start_time_history().back();
     get_start_time_history().pop_back();
-    pqassert(get_max_call_times_for_always_show_info_history().size() >= 1);
+    qassert(get_max_call_times_for_always_show_info_history().size() >= 1);
     max_call_times_for_always_show_info() =
         get_max_call_times_for_always_show_info_history().back();
     get_max_call_times_for_always_show_info_history().pop_back();
     std::vector<std::vector<TimerInfo> >& tdb_history =
         get_timer_database_history();
     std::vector<TimerInfo>& tdb = get_timer_database();
-    pqassert(tdb_history.size() >= 1);
-    pqassert(tdb.size() >= tdb_history.back().size());
+    qassert(tdb_history.size() >= 1);
+    qassert(tdb.size() >= tdb_history.back().size());
     for (long i = 0; i < (long)tdb_history.back().size(); ++i) {
       tdb[i].merge(tdb_history.back()[i]);
     }
@@ -567,23 +594,21 @@ struct API Timer {
     get_start_time();
     initialize_papi();
     info_index = -1;
-    isRunning = 0;
+    is_running = 0;
   }
   void init(const std::string& fname_str)
   {
+    std::map<std::string, long>& tiim = get_timer_info_index_map();
     std::vector<TimerInfo>& tdb = get_timer_database();
-    const long size = tdb.size();
-    for (long i = 0; i < size; i++) {
-      if (fname_str == tdb[i].fname) {
-        info_index = i;
-        return;
-      }
+    if (has(tiim, fname_str)) {
+      info_index = tiim[fname_str];
+    } else {
+      info_index = tdb.size();
+      tiim[fname_str] = info_index;
+      TimerInfo info;
+      info.fname = fname_str;
+      tdb.push_back(info);
     }
-    info_index = tdb.size();
-    TimerInfo info;
-    tdb.push_back(info);
-    tdb[info_index].init();
-    tdb[info_index].fname = fname_str;
   }
   void init(const std::string& cname_str, const std::string& fname_str)
   {
@@ -597,18 +622,18 @@ struct API Timer {
   void start(bool verbose = false)
   {
     get_timer_stack().push_back(info_index);
-    if (isRunning > 0) {
-      isRunning += 1;
+    if (is_running > 0) {
+      is_running += 1;
       return;
-    } else if (isRunning == 0) {
-      isRunning = 1;
+    } else if (is_running == 0) {
+      is_running = 1;
     } else {
       TimerInfo& info = get_timer_database()[info_index];
       info.show_avg_always("debug", max_function_name_length_shown());
-      displayln(ssprintf("%s::%s ERROR: isRunning=%d", cname,
-                         info.fname.c_str(), isRunning));
+      displayln(ssprintf("%s::%s ERROR: is_running=%d", cname,
+                         info.fname.c_str(), is_running));
       Timer::display_stack();
-      pqassert(false);
+      qassert(false);
     }
     TimerInfo& info = get_timer_database()[info_index];
     info.call_times++;
@@ -629,23 +654,23 @@ struct API Timer {
   {
     TimerInfo& info = get_timer_database()[info_index];
     std::vector<long>& t_stack = get_timer_stack();
-    pqassert(not t_stack.empty());
+    qassert(not t_stack.empty());
     if (not(t_stack.back() == info_index)) {
       displayln(ssprintf("%s::%s ERROR: stack is corrupted", cname,
                          info.fname.c_str()));
       Timer::display_stack();
-      pqassert(false);
+      qassert(false);
     }
     t_stack.pop_back();
-    if (isRunning <= 0) {
+    if (is_running <= 0) {
       info.show_avg_always("debug", max_function_name_length_shown());
-      displayln(ssprintf("%s::%s ERROR: isRunning=%d", cname,
-                         info.fname.c_str(), isRunning));
+      displayln(ssprintf("%s::%s ERROR: is_running=%d", cname,
+                         info.fname.c_str(), is_running));
       Timer::display_stack();
-      pqassert(false);
+      qassert(false);
     }
-    isRunning -= 1;
-    if (isRunning != 0) {
+    is_running -= 1;
+    if (is_running != 0) {
       return;
     }
     stop_time = get_time();
@@ -693,7 +718,7 @@ struct API Timer {
     }
   }
   //
-  API static void display(const std::string& str = "")
+  API static void display(const std::string& tag = "")
   {
     double total_time = get_total_time();
     const std::vector<TimerInfo>& tdb = get_timer_database();
@@ -708,7 +733,7 @@ struct API Timer {
     displayln_info(ssprintf(
         "Timer::display-start: %s (level=%ld) fname : time%% number of calls; "
         "Avg,Tot sec; Avg,Tot flops; Gflops",
-        str.c_str(), (long)tdb_history.size()));
+        tag.c_str(), (long)tdb_history.size()));
     const long dbsize = db.size();
     for (long i = 0; i < dbsize; i++) {
       if (db[i]->call_times > 0) {
@@ -718,7 +743,7 @@ struct API Timer {
     displayln_info(ssprintf(
         "Timer::display-end:   %s (level=%ld) --------------------- total %.4E "
         "sec ----------------------",
-        str.c_str(), (long)tdb_history.size(), total_time));
+        tag.c_str(), (long)tdb_history.size(), total_time));
   }
   //
   API static void autodisplay(const double time)
@@ -814,7 +839,7 @@ inline void Display(const char* cname, const char* fname, const char* format,
   va_list args;
   va_start(args, format);
   char* str;
-  pqassert(vasprintf(&str, format, args) >= 0);
+  qassert(vasprintf(&str, format, args) >= 0);
   display(ssprintf("%s::%s : %s", cname, fname, str));
   std::free(str);
 }
@@ -829,13 +854,9 @@ inline void DisplayInfo(const char* cname, const char* fname,
   va_list args;
   va_start(args, format);
   char* str;
-  pqassert(vasprintf(&str, format, args) >= 0);
+  qassert(vasprintf(&str, format, args) >= 0);
   display_info(ssprintf("%s::%s : %s", cname, fname, str));
   std::free(str);
 }
 
 }  // namespace qlat
-
-#ifndef USE_NAMESPACE
-using namespace qlat;
-#endif
